#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>

#define NOTHING 0
#define MESSAGE 1
#define QUESTION 2
#define PROGRESS 3

#define AUTO_CLOSE_OPTION 100

GtkWidget *progress, *yes, *no;
double current_number;
GIOChannel *g_stdin;
int auto_close;

void yes_clicked(GtkWidget * widget, GtkWidget * mywin)
{
    gtk_main_quit();
    exit(0);
}

void no_clicked(GtkWidget * widget, GtkWidget * mywin)
{
    gtk_main_quit();
    exit(1);
}

void update_progressbar()
{
    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress), current_number/100.0);
    if(current_number < 100.0) {
        if(yes)
            gtk_widget_set_sensitive(yes, FALSE);
    } else {
            if(auto_close == TRUE)
            yes_clicked(NULL, NULL);
        if(yes) {
            gtk_widget_set_sensitive(yes, TRUE);
            gtk_widget_grab_focus(yes);
        }
    }
}

void update_progressbar_from_channel(GIOChannel * channel)
{
    GString *buf;
    buf = g_string_new("");
    g_io_channel_read_line_string(channel, buf, NULL, NULL);
    current_number = atof(buf->str);
    update_progressbar();
    g_string_free(buf, TRUE);
    g_io_channel_flush(g_stdin, NULL);
}

void usage(char * progname) {
    printf("Usage: %s --message|--question|--progress [mode specific options] \"message\"\n", progname);
    printf("Progress mode options: --auto-close\n");
    exit(1);
}

#define USAGE() usage(argv[0])

int main(int argc, char ** argv){
    GtkWidget *mywin, *vbox, *hbox, *label, *spacer, *big_hbox, *ospacer, *o2spacer;
    int mode;
    int retval;
    char * message, * yestext, * notext;
    int has_no_button, has_yes_button;
    mode = NOTHING;
    has_no_button = TRUE;
    has_yes_button = TRUE;
    auto_close = FALSE;

    // Option processing
    struct option longopts[] = {
        {"message", 0, 0, MESSAGE},
        {"question", 0, 0, QUESTION},
        {"progress", 0, 0, PROGRESS},
        {"auto-close", 0, 0, AUTO_CLOSE_OPTION},
        {0,0,0,0}
    };
    while(1) {
        retval = getopt_long(argc, argv, "", longopts, NULL);
        if(retval == -1)
            break;
        switch(retval) {
        case MESSAGE:
        case QUESTION:
        case PROGRESS:
            if(mode != NOTHING)
                USAGE();
            mode = retval;
            break;
        case AUTO_CLOSE_OPTION:
            auto_close = TRUE;
            break;
        default:
            USAGE();
            break;
        }
    }
    if(mode == NOTHING)
        USAGE();
    if((argc - optind) != 1)
        USAGE();
    if(auto_close == TRUE && mode != PROGRESS)
        USAGE();
    message = argv[optind];
    switch(mode) {
    case MESSAGE:
        yestext=g_strdup("OK");
        has_no_button = FALSE;
        break;
    case QUESTION:
        yestext=g_strdup("Yes");
        notext=g_strdup("No");
        break;
    case PROGRESS:
        if(auto_close == TRUE)
            has_yes_button = FALSE;
        else
            yestext=g_strdup("OK");
        notext=g_strdup("Cancel");
        break;
    }

    // Set up the basic stuff
    gtk_init(&argc, &argv);

    // Theme
    char * ldm_theme;
    gchar * ldm_gtkrc;
    ldm_theme = getenv("LDM_THEME");
    if (ldm_theme)
    {
        if(*ldm_theme == '/')
            ldm_gtkrc = g_strconcat(ldm_theme, "/greeter-gtkrc", NULL);
        else
            ldm_gtkrc = g_strconcat(LDM_THEME_DIR, ldm_theme, "/greeter-gtkrc", NULL);
    } else
        ldm_gtkrc = g_strconcat(LDM_THEME_DIR, "default", "/greeter-gtkrc", NULL);
    gtk_rc_add_default_file(ldm_gtkrc);
    g_free(ldm_gtkrc);

    // Finish setting up the basic stuff
    mywin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_position(GTK_WINDOW(mywin), GTK_WIN_POS_CENTER_ALWAYS);
    vbox = gtk_vbox_new(FALSE, 0);
    hbox = gtk_hbox_new(FALSE, 3);
    spacer = gtk_label_new("");
    ospacer = gtk_label_new("");
    o2spacer = gtk_label_new("");
    gtk_widget_set_size_request(ospacer, 0, 8);
    gtk_widget_set_size_request(o2spacer, 0, 8);
    gtk_box_pack_start(GTK_BOX(hbox), spacer, TRUE, FALSE, 0);

    // Make the yes button
    if(has_yes_button == TRUE) {
        yes = gtk_button_new_with_label(yestext);
        gtk_button_set_image(GTK_BUTTON(yes), gtk_button_get_image(GTK_BUTTON(gtk_button_new_from_stock("gtk-ok"))));
        gtk_widget_set_size_request(yes, 80, 30);
        g_signal_connect(G_OBJECT(yes), "clicked", G_CALLBACK(yes_clicked), mywin);
        gtk_box_pack_start(GTK_BOX(hbox), yes, FALSE, FALSE, 0);
    }

    // Make the no button
    if(has_no_button == TRUE) {
        no = gtk_button_new_with_label(notext);
        gtk_button_set_image(GTK_BUTTON(no), gtk_button_get_image(GTK_BUTTON(gtk_button_new_from_stock("gtk-cancel"))));
        gtk_widget_set_size_request(no, 80, 30);
        g_signal_connect(G_OBJECT(no), "clicked", G_CALLBACK(no_clicked), mywin);
        gtk_box_pack_start(GTK_BOX(hbox), no, FALSE, FALSE, 0);
    }

    // Create the label
    label = gtk_label_new(message);
    gtk_widget_set_size_request(label, 300, -1);
    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 10);

    if(mode == PROGRESS) {
        // Make the progress bar
        progress = gtk_progress_bar_new();
        gtk_widget_set_size_request(progress, 300, -1);
        gtk_box_pack_start(GTK_BOX(vbox), progress, FALSE, FALSE, 0);
        gtk_box_pack_start(GTK_BOX(vbox), o2spacer, FALSE, FALSE, 0);

        // Set the starting value
        current_number = 0.0;
        update_progressbar();

        // Set up the hook that listens for input
        g_stdin = g_io_channel_unix_new(0);
        g_io_add_watch(g_stdin, G_IO_IN, (GIOFunc) update_progressbar_from_channel, g_stdin);
    }

    // More basic GTK stuff
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), ospacer, FALSE, FALSE, 0);
    big_hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(big_hbox), vbox, FALSE, FALSE, 10);
    gtk_container_add(GTK_CONTAINER(mywin), big_hbox);
    gtk_widget_show_all(mywin);
    gtk_main();

    // Clean up
    if(has_yes_button == TRUE)
        g_free(yestext);
    if(has_no_button == TRUE)
        g_free(notext);
    return 0;
}
